#include <cstdio>
#include <vector>
#include <algorithm>
#include <cmath>
using namespace std;

typedef long long ll;

// simple point class
struct point{
	int x, y;
	point(int x, int y): x(x), y(y){}
	bool operator==(const point& rhs) const{
		return x==rhs.x && y==rhs.y;
	}
};

// simple fraction class
struct frac{
	ll num, den;
	frac(ll num, ll den): num(num), den(den){}
	bool operator<(const frac& rhs) const{
		return num*rhs.den < rhs.num*den;
	}
};

// comparator class for sorting points with respect to some vector
struct compare{
	int dx, dy;
	compare(const point& p1, const point& p2): dx(p2.x-p1.x), dy(p2.y-p1.y){}
	bool operator()(const point& p1, const point& p2) const{
		int dx2=p2.x-p1.x, dy2=p2.y-p1.y;
		return dx*dx2 + dy*dy2 > 0;
	}
};

// trapezoid area
int trap(point a, point b){
	return (b.x-a.x)*(b.y+a.y);
}

// triangle area
int area(point a, point b, point c){
	return trap(a, b)+trap(b, c)+trap(c, a);
}

// square of the distance from point c to line ab, with original sign
frac height(point a, point b, point c){
	ll ar=area(a, b, c);
	int dx=b.x-a.x, dy=b.y-a.y;
	return frac(llabs(ar)*ar, dx*dx+dy*dy);
}

// solve method - returns a polygonal segment beginning at _start_ and ending
// at _end_, with all points from _inside_ inside the resulting polygon and 
// points from _outside_ outside.  All points from _inside_ and _outside_ must
// lie on the same side of the line _start_end_ and must lie strictly between
// the lines perpendicular to _start_end_ and passing through _start_ or
// _end_.
vector<point> solve(point start, point end, vector<point> inside, vector<point> outside){
	// begin with a convex hull algorithm, using _start_end_ as the reference line
	sort(inside.begin(), inside.end(), compare(start, end));
	vector<point> hull;
	hull.push_back(start);
	inside.push_back(end);
	for(int i=0; i<inside.size(); i++){
		while(hull.size()>1 && area(hull[hull.size()-2], hull[hull.size()-1], inside[i]) <= 0)
			hull.pop_back();
		hull.push_back(inside[i]);
	}
	// base case
	if(hull.size() == 2)
		return hull;
	// for each point inside the convex hull, find the edge closest to it
	vector<vector<point> > nextin(hull.size()-1), nextout(hull.size()-1);
	for(int i=0; i<inside.size(); i++){
		if(inside[i]==start || inside[i]==end)
			continue;
		bool found=false;
		int best=-1;
		frac bestcost(1000000000, 1);
		for(int j=1; j<hull.size(); j++){
			if(inside[i]==hull[j]){
				found=true;
				break;
			}
			frac cost=height(hull[j-1], hull[j], inside[i]);
			if(cost<bestcost){
				bestcost=cost;
				best=j-1;
			}
		}
		if(!found)
			nextin[best].push_back(inside[i]);
	}
	for(int i=0; i<outside.size(); i++){
		if(outside[i]==start || outside[i]==end)
			continue;
		int best=-1;
		frac bestcost(1000000000, 1);
		for(int j=1; j<hull.size(); j++){
			frac cost=height(hull[j-1], hull[j], outside[i]);
			if(cost<bestcost){
				bestcost=cost;
				best=j-1;
			}
		}
		if(bestcost.num>=0)
			nextout[best].push_back(outside[i]);
	}
	// now, for each edge of the convex hull, call this method recursively,
	// and combine the individual results into a single polygon.  Note that
	// inside and outside are reversed in the recursive call.
	vector<point> res;
	for(int i=0; i<hull.size()-1; i++){
		vector<point> next=solve(hull[i+1], hull[i], nextout[i], nextin[i]);
		res.insert(res.end(), next.rbegin(), next.rend());
		res.pop_back();
	}
	res.push_back(hull.back());
	return res;
}

int main(){
	int N;
	// red points rebytelicans, blue points decocrats
	vector<point> red, blue;
	// read the input
	scanf("%d", &N);
	int minx=1000, maxx=0, maxy=0;
	while(N--){
		int x, y, k;
		scanf("%d %d %d", &x, &y, &k);
		(k==1000 ? blue : red).push_back(point(x, y));
		minx=min(minx, x);
		maxx=max(maxx, x);
		maxy=max(maxy, y);
	}
	// call the solve method.
	vector<point> res=solve(point(maxx+1, maxy+1), point(minx-1, maxy+1), red, blue);
	// print the result
	printf("%d\n", res.size());
	for(int i=0; i<res.size(); i++){
		printf("%d %d\n", res[i].x, res[i].y);
	}
	return 0;
}
